# Setup Keycloak als SAML IDP für den Antragsraum

## Voraussetzungen:

Damit der Antragsraum Keycloak als SAML IDP verwenden kann, müssen ersteinmal die passenden Keys und Zertifikate
erstellt werden.

Dazu müssen Zertifikate mit privaten Keys erstellt werden:

Beispiel:

```
openssl req -newkey rsa:2048 -nodes -keyout bayernid-kc-sign.key -x509 -days 365 -out bayernid-test-sign.crt
```

## Realm erstellen

In Keycloak einen neuen Realm erstellen, der für den Antragsraum benutzt werden soll.

Als "Realm name" wird z.B. "by-antragsraum-idp" verwendet.
Der Realmname ist dann auch Id des realms.

Nach Erstellung des Realms findet man unter
"Realm setting" → "General" Tab den Punkt "Endpoints", dort ist der Link zu den Metadaten für den SAML Login.

## Client erstellen

"Clients" → "Clients list" Tab den Button "Create Client" drücken.

Diese Werte eintragen:

"Client type" ist "SAML"
"Client ID" ist am besten die EntityID aus den Metadaten.
"Name" ist z.B. "Antragsraum"
"Description" ist z.B. "SAML Client für Antragsraum"

## Client Einstellungen

Unter "Clients" den erstellten Client auswählen
und im Tab "Settings" folgendes Eintragen:

| Feld                            | Werte                                                                                                 |
|---------------------------------|-------------------------------------------------------------------------------------------------------|
| Root URL                        | https://dev.antragsraum.de                                                                            |
| Home URL                        | https://dev.antragsraum.de                                                                            |
| Valid redirect URIs             | http://localhost:8080/login/saml2/sso/bayernid, https://dev.antragsraum.de/login/saml2/sso/bayernid   |
| Valid post logout redirect URIs | http://localhost:8080/logout/saml2/sso/bayernid, https://dev.antragsraum.de/logout/saml2/sso/bayernid |

Die anderen Einstellungen bleiben beim Default.

## User erstellen

Unter "Users" im Tab "User list" auf den Button "Add user" drücken und den User erstellen.
Beispiel Werte sind:

| Feld                  | Wert   |
|-----------------------|--------|
| Required user actions | leer   |
| Username              | test   |
| First name            | Test   |
| Last name             | Level1 |

Nachdem der User erstellt wurde den User anklicken und die Attribute eintrage:

| Feld           | Wert                                 |
|----------------|--------------------------------------|
| trustlevel     | STORK-QAA-Level-1                    |
| postkorbHandle | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 |
| vorname        | Test                                 |
| nachname       | Level1                               |

## Attribute mapping konfigurieren

Das Mapping muss nur einmal konfiguriert werden und wird dann für alle User angewendet.

Unter "Client scopes" "role_list" auswählen dann den Tab "Mappers" auswählen.

Dort die Mappings der User Attributes zu SAML Attributes konfigurieren.

Dazu "Add Mappers" -> "By Configuration" auswählen

Postkorbhandle Mapper:

| Feld                | Wert             |
|---------------------|------------------|
| Mapper type         | User Attribute   |
| Name                | PostkorbHandle   |
| User Attribute      | postkorbHandle   |
| Friendly Name       | PostkorbHandle   |
| SAML Attribute Name | urn:oid:2.5.4.18 |

Trustlevel Mapper:

| Feld                | Wert                             |
|---------------------|----------------------------------|
| Mapper type         | User Attribute                   |
| Name                | TrustLevel                       |
| User Attribute      | trustlevel                       |
| Friendly Name       | TrustLevel                       |
| SAML Attribute Name | urn:oid:1.2.40.0.10.2.1.1.261.94 |

! Wichtig die 'User Attribute' Werte müssen zu den Namen der User Attribute passen.

## Erstelle Keys importieren

Das weiter oben erstelle Zertifikat muss noch in den Client importiert werden.

Dazu unter "Clients" den SAML Client anwählen und dann den Tab "Keys" öffen.
Dort bei "Certificate" den Button "Import Key" drücken.
Als "Archive Format" "Certificate PEM" auswählen und dann mit "Browse" die erstellte crt Datei auswählen.
Anschließend den "Import" Button drücken.

### Users

| login      | Password | Trustlevel        | Postfachhandle                       | BayernId User                                       |
|------------|----------|-------------------|--------------------------------------|-----------------------------------------------------|
| test       | test     | STORK-QAA-Level-1 | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 | ozg-kop                                             |
| testLevel1 | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-1 | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 |                                                     |
| testLevel2 | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-2 | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 |                                                     |
| testLevel3 | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-3 | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 |                                                     |
| testLevel4 | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-4 | 28721c6f-b78f-4d5c-a048-19fd2fc429d2 |                                                     |
| ohl        | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-3 | 8456e4d4-87fe-4351-951d-8c77b19b5d04 | Zertifikat(xOCA0002_elsterOrg_08.05.2024_16.48.pfx) |
| ohllevel1  | Y9nk43yrQ_zzIPpfFU-I | STORK-QAA-Level-1 | 8456e4d4-87fe-4351-951d-8c77b19b5d04 | bitherstone                                         |
