/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.UUID;

public class NachrichtTestFactory {
	public static final String ID = UUID.randomUUID().toString();
	public static final String NACHRICHT_EVENT_ID = UUID.randomUUID().toString();
	public static final String POSTFACH_ID = UUID.randomUUID().toString();
	public static final String VORGANG_ID = UUID.randomUUID().toString();
	public static final String TEXT = """
	  Lorem ipsum dolor sit amet, consetetur sadipscing elitr,
	   sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat,
	   sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.
	   
	   Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.
	   Lorem ipsum dolor sit amet, consetetur sadipscing elitr.
	  """;
	public static final String VORGANG_TITLE = "Vorgang Test ";
	public static final String ATTACHMENT_ID_1 = "6358fd3f46811d04010f44c8";
	public static final String ATTACHMENT_ID_2 = "6358fd4146811d04010f44d0";
	public static final String DATE_STRING = "2022-10-26T09:26:25";
	public static final long DATE = LocalDateTime.parse(DATE_STRING).toEpochSecond(ZoneOffset.UTC);
	public static final ReplyOption REPLY_ALLOWED = ReplyOption.ALLOWED;
	public static final String REPLY_TO_NACHRICHT_ID = UUID.randomUUID().toString();
	public static final boolean ACCESSIBLE = true;
	public static final String TRUST_LEVEL = "LOW";
	public static final String STATUS = "ANSWERED";

	static Nachricht createNachricht() {
		return createNachrichtBuilder().build();
	}

	static Nachricht.NachrichtBuilder createNachrichtBuilder() {
		return Nachricht.builder()
		  .id(ID)
		  .nachrichtEventId(NACHRICHT_EVENT_ID)
		  .message(TEXT)
		  .title(VORGANG_TITLE)
		  .vorgangId(VORGANG_ID)
		  .postfachId(POSTFACH_ID)
		  .replyOption(REPLY_ALLOWED)
		  .replyNachrichten(List.of(ReplyNachricht.builder().nachrichtEventId(NACHRICHT_EVENT_ID).build()))
		  .status(STATUS)
		  .answeredAt(DATE)
		  .sendAt(DATE)
		  .trustLevel(TRUST_LEVEL)
		  .accessible(ACCESSIBLE);
	}

	static NachrichtHeader createNachrichtHeader() {
		return createNachrichtHeaderBuilder().build();
	}

	static NachrichtHeader.NachrichtHeaderBuilder createNachrichtHeaderBuilder() {
		return NachrichtHeader.builder()
		  .id(ID)
		  .nachrichtEventId(NACHRICHT_EVENT_ID)
		  .vorgangId(VORGANG_ID)
		  .postfachId(POSTFACH_ID)
		  .title(VORGANG_TITLE)
		  .replyOption(REPLY_ALLOWED)
		  .answeredAt(DATE)
		  .sendAt(DATE)
		  .trustLevel(TRUST_LEVEL)
		  .accessible(ACCESSIBLE)
		  .status(STATUS);
	}
}
